@@ -330,6 +330,29 @@ def invoke_rest_api(api_id, stage, method, invocation_path, data, headers, path=
             # TODO apply response template..?
             return result
 
+        elif 'states:action/' in uri:
+            if uri.endswith('states:action/StartExecution'):
+                action = 'StartExecution'
+            decoded_data = data.decode()
+            if 'stateMachineArn' in decoded_data and 'input' in decoded_data:
+                payload = json.loads(decoded_data)
+            elif APPLICATION_JSON in integration.get('requestTemplates', {}):
+                template = integration['requestTemplates'][APPLICATION_JSON]
+                payload = aws_stack.render_velocity_template(template, data, as_json=True)
+            client = aws_stack.connect_to_service('stepfunctions')
+
+            kwargs = {'name': payload['name']} if 'name' in payload else {}
+            result = client.start_execution(stateMachineArn=payload['stateMachineArn'],
+                            input=payload['input'], **kwargs)
+            response = requests_response(
+                content={
+                    'executionArn': result['executionArn'],
+                    'startDate': str(result['startDate'])
+                },
+                headers=aws_stack.mock_aws_request_headers()
+            )
+            return response
+
         if method == 'POST':
             if uri.startswith('arn:aws:apigateway:') and ':sqs:path' in uri:
                 template = integration['requestTemplates'][APPLICATION_JSON]
@@ -417,7 +440,7 @@ def invoke_rest_api(api_id, stage, method, invocation_path, data, headers, path=
 def get_lambda_event_request_context(method, path, data, headers, integration_uri=None, resource_id=None):
     _, stage, relative_path_w_query_params = get_api_id_stage_invocation_path(path)
     relative_path, query_string_params = extract_query_string_params(path=relative_path_w_query_params)
-    source_ip = headers['X-Forwarded-For'].split(',')[-2].strip()
+    source_ip = headers.get('X-Forwarded-For', ',').split(',')[-2].strip()
     integration_uri = integration_uri or ''
     account_id = integration_uri.split(':lambda:path')[-1].split(':function:')[0].split(':')[-1]
     request_context = {
@@ -430,7 +453,7 @@ def get_lambda_event_request_context(method, path, data, headers, integration_ur
         'identity': {
             'accountId': account_id,
             'sourceIp': source_ip,
-            'userAgent': headers['User-Agent'],
+            'userAgent': headers.get('User-Agent'),
         },
         'httpMethod': method,
         'protocol': 'HTTP/1.1',
