@@ -67,6 +67,10 @@ EVENT_SOURCE_SQS = 'aws:sqs'
 # IP address of main Docker container (lazily initialized)
 DOCKER_MAIN_CONTAINER_IP = None
 
+# whether to use our custom Java executor, or the default from lambci
+# TODO: deprecated, should be removed in the future
+USE_CUSTOM_JAVA_EXECUTOR = False
+
 
 def get_from_event(event, key):
     try:
@@ -179,8 +183,21 @@ class LambdaExecutor(object):
         pass
 
     def run_lambda_executor(self, cmd, event=None, func_details=None, env_vars={}):
-        process = run(cmd, asynchronous=True, stderr=subprocess.PIPE, outfile=subprocess.PIPE,
-                      env_vars=env_vars, stdin=True)
+        kwargs = {'stdin': True, 'inherit_env': True, 'asynchronous': True}
+
+        is_provided = func_details.runtime.startswith(LAMBDA_RUNTIME_PROVIDED)
+        if func_details and is_provided and env_vars.get('DOCKER_LAMBDA_USE_STDIN') == '1':
+            # Note: certain "provided" runtimes (e.g., Rust programs) can block when we pass in
+            # the event payload via stdin, hence we rewrite the command to "echo ... | ..." below
+            env_vars = {
+                'PATH': env_vars.get('PATH') or os.environ.get('PATH', ''),
+                'AWS_LAMBDA_EVENT_BODY': to_str(event),
+                'DOCKER_LAMBDA_USE_STDIN': '1'
+            }
+            event = None
+            cmd = re.sub(r'(.*)(%s\s+(run|start))' % self._docker_cmd(), r'\1echo $AWS_LAMBDA_EVENT_BODY | \2', cmd)
+
+        process = run(cmd, env_vars=env_vars, stderr=subprocess.PIPE, outfile=subprocess.PIPE, **kwargs)
         result, log_output = process.communicate(input=event)
         try:
             result = to_str(result).strip()
@@ -210,9 +227,7 @@ class LambdaExecutor(object):
 
 
 class ContainerInfo:
-    """
-    Contains basic information about a docker container.
-    """
+    """ Contains basic information about a docker container. """
     def __init__(self, name, entry_point):
         self.name = name
         self.entry_point = entry_point
@@ -254,6 +269,7 @@ class LambdaExecutorContainers(LambdaExecutor):
         main_endpoint = get_main_endpoint_from_container()
 
         environment['LOCALSTACK_HOSTNAME'] = main_endpoint
+        environment['EDGE_PORT'] = str(config.EDGE_PORT)
         environment['_HANDLER'] = handler
         if os.environ.get('HTTP_PROXY'):
             environment['HTTP_PROXY'] = os.environ['HTTP_PROXY']
@@ -272,8 +288,8 @@ class LambdaExecutorContainers(LambdaExecutor):
         command = ''
         events_file = ''
 
-        # if running a Java Lambda, set up classpath arguments
-        if is_java_lambda(runtime):
+        if USE_CUSTOM_JAVA_EXECUTOR and is_java_lambda(runtime):
+            # if running a Java Lambda with our custom executor, set up classpath arguments
             java_opts = Util.get_java_opts()
             stdin = None
             # copy executor jar into temp directory
@@ -423,6 +439,7 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
                     ' -e AWS_LAMBDA_EVENT_BODY="$AWS_LAMBDA_EVENT_BODY"'
                     ' -e HOSTNAME="$HOSTNAME"'
                     ' -e LOCALSTACK_HOSTNAME="$LOCALSTACK_HOSTNAME"'
+                    ' -e EDGE_PORT="$EDGE_PORT"'
                     '  %s'  # env_vars
                     '  %s'  # network
                     '  %s'  # dns
@@ -483,9 +500,7 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
 
             if status == 1:
                 LOG.debug('Stopping container: %s' % container_name)
-                cmd = (
-                    '%s stop -t0 %s'
-                ) % (docker_cmd, container_name)
+                cmd = '%s stop -t0 %s' % (docker_cmd, container_name)
 
                 LOG.debug(cmd)
                 run(cmd, asynchronous=False, stderr=subprocess.PIPE, outfile=subprocess.PIPE)
@@ -494,9 +509,7 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
 
             if status == -1:
                 LOG.debug('Removing container: %s' % container_name)
-                cmd = (
-                    '%s rm %s'
-                ) % (docker_cmd, container_name)
+                cmd = '%s rm %s' % (docker_cmd, container_name)
 
                 LOG.debug(cmd)
                 run(cmd, asynchronous=False, stderr=subprocess.PIPE, outfile=subprocess.PIPE)
@@ -767,9 +780,9 @@ class Util:
             opts = opts.replace('_debug_port_', ('%s' % cls.debug_java_port))
         else:
             # Parse the debug port from opts
-            m = re.match('.*address=(\\d+).*', opts)
+            m = re.match('.*address=(.+:)?(\\d+).*', opts)
             if m is not None:
-                cls.debug_java_port = m.groups()[0]
+                cls.debug_java_port = m.groups()[1]
 
         return opts
 
