@@ -5,8 +5,8 @@ import traceback
 from localstack import config
 from localstack.services import install
 from localstack.utils.aws import aws_stack
-from localstack.utils.common import is_root, mkdir, chmod_r, rm_rf, get_free_tcp_port
-from localstack.services.infra import get_service_protocol, start_proxy_for_service, do_run
+from localstack.utils.common import is_root, mkdir, chmod_r, rm_rf, get_free_tcp_port, get_service_protocol
+from localstack.services.infra import start_proxy_for_service, do_run
 
 LOG = logging.getLogger(__name__)
 
@@ -26,7 +26,8 @@ def stop_elasticsearch():
         return
     LOG.info('Terminating Elasticsearch instance, as all clusters have been removed')
     thread.stop()
-    STATE['_proxy_'].stop()
+    if STATE['_proxy_']:
+        STATE['_proxy_'].stop()
     del STATE['_thread_']
     del STATE['_proxy_']
 
