@@ -17,7 +17,7 @@ from localstack.utils.analytics import event_publisher
 from localstack.utils.aws import aws_stack
 from localstack.utils.aws.aws_responses import response_regex_replace
 from localstack.utils.aws.dead_letter_queue import sns_error_to_dead_letter_queue
-from localstack.utils.common import timestamp_millis, short_uid, to_str, to_bytes
+from localstack.utils.common import timestamp_millis, short_uid, to_str, to_bytes, start_thread
 from localstack.utils.persistence import PersistingProxyListener
 
 # set up logger
@@ -120,7 +120,7 @@ class ProxyListenerSNS(PersistingProxyListener):
                 # No need to create a topic to send SMS or single push notifications with SNS
                 # but we can't mock a sending so we only return that it went well
                 if 'PhoneNumber' not in req_data and 'TargetArn' not in req_data:
-                    if topic_arn not in SNS_SUBSCRIPTIONS.keys():
+                    if topic_arn not in SNS_SUBSCRIPTIONS:
                         return make_error(code=404, code_string='NotFound', message='Topic does not exist')
 
                 message_id = publish_message(topic_arn, req_data)
@@ -144,6 +144,7 @@ class ProxyListenerSNS(PersistingProxyListener):
             elif req_action == 'CreateTopic':
                 topic_arn = aws_stack.sns_topic_arn(req_data['Name'][0])
                 tag_resource_success = self._extract_tags(topic_arn, req_data, True)
+                SNS_SUBSCRIPTIONS[topic_arn] = SNS_SUBSCRIPTIONS.get(topic_arn) or []
                 # in case if there is an error it returns an error , other wise it will continue as expected.
                 if not tag_resource_success:
                     return make_error(code=400, code_string='InvalidParameter',
@@ -228,7 +229,6 @@ class ProxyListenerSNS(PersistingProxyListener):
             if req_action == 'CreateTopic' and response.status_code < 400:
                 response_data = xmltodict.parse(response.content)
                 topic_arn = response_data['CreateTopicResponse']['CreateTopicResult']['TopicArn']
-                do_create_topic(topic_arn)
                 # publish event
                 event_publisher.fire_event(
                     event_publisher.EVENT_SNS_CREATE_TOPIC,
@@ -257,17 +257,7 @@ def unsubscribe_sqs_queue(queue_url):
                 subscriptions.remove(subscriber)
 
 
-def publish_message(topic_arn, req_data, subscription_arn=None, skip_checks=False):
-    message = req_data['Message'][0]
-    message_id = str(uuid.uuid4())
-
-    LOG.debug('Publishing message to TopicArn: %s | Message: %s' % (topic_arn, message))
-
-    if topic_arn and ':endpoint/' in topic_arn:
-        # cache messages published to platform endpoints
-        cache = PLATFORM_ENDPOINT_MESSAGES[topic_arn] = PLATFORM_ENDPOINT_MESSAGES.get(topic_arn) or []
-        cache.append(req_data)
-
+def message_to_subscribers(message_id, message, topic_arn, req_data, subscription_arn=None, skip_checks=False):
     subscriptions = SNS_SUBSCRIPTIONS.get(topic_arn, [])
     for subscriber in list(subscriptions):
         if subscription_arn not in [None, subscriber['SubscriptionArn']]:
@@ -375,16 +365,25 @@ def publish_message(topic_arn, req_data, subscription_arn=None, skip_checks=Fals
         else:
             LOG.warning('Unexpected protocol "%s" for SNS subscription' % subscriber['Protocol'])
 
+
+def publish_message(topic_arn, req_data, subscription_arn=None, skip_checks=False):
+    message = req_data['Message'][0]
+    message_id = str(uuid.uuid4())
+
+    if topic_arn and ':endpoint/' in topic_arn:
+        # cache messages published to platform endpoints
+        cache = PLATFORM_ENDPOINT_MESSAGES[topic_arn] = PLATFORM_ENDPOINT_MESSAGES.get(topic_arn) or []
+        cache.append(req_data)
+
+    LOG.debug('Publishing message to TopicArn: %s | Message: %s' % (topic_arn, message))
+    start_thread(
+        lambda _: message_to_subscribers(message_id, message, topic_arn, req_data, subscription_arn, skip_checks))
     return message_id
 
 
-def do_create_topic(topic_arn):
-    if topic_arn not in SNS_SUBSCRIPTIONS:
-        SNS_SUBSCRIPTIONS[topic_arn] = []
-
-
 def do_delete_topic(topic_arn):
     SNS_SUBSCRIPTIONS.pop(topic_arn, None)
+    SNS_TAGS.pop(topic_arn, None)
 
 
 def do_confirm_subscription(topic_arn, token):
@@ -394,9 +393,11 @@ def do_confirm_subscription(topic_arn, token):
 
 
 def do_subscribe(topic_arn, endpoint, protocol, subscription_arn, attributes, filter_policy=None):
+    topic_subs = SNS_SUBSCRIPTIONS[topic_arn] = SNS_SUBSCRIPTIONS.get(topic_arn) or []
+
     # An endpoint may only be subscribed to a topic once. Subsequent
     # subscribe calls do nothing (subscribe is idempotent).
-    for existing_topic_subscription in SNS_SUBSCRIPTIONS.get(topic_arn, []):
+    for existing_topic_subscription in topic_subs:
         if existing_topic_subscription.get('Endpoint') == endpoint:
             return
 
@@ -409,9 +410,9 @@ def do_subscribe(topic_arn, endpoint, protocol, subscription_arn, attributes, fi
         'FilterPolicy': filter_policy
     }
     subscription.update(attributes)
-    SNS_SUBSCRIPTIONS[topic_arn].append(subscription)
+    topic_subs.append(subscription)
 
-    if subscription_arn not in SUBSCRIPTION_STATUS.keys():
+    if subscription_arn not in SUBSCRIPTION_STATUS:
         SUBSCRIPTION_STATUS[subscription_arn] = {}
 
     SUBSCRIPTION_STATUS[subscription_arn].update(
@@ -436,9 +437,9 @@ def do_subscribe(topic_arn, endpoint, protocol, subscription_arn, attributes, fi
 
 
 def do_unsubscribe(subscription_arn):
-    for topic_arn in SNS_SUBSCRIPTIONS:
+    for topic_arn, existing_subs in SNS_SUBSCRIPTIONS.items():
         SNS_SUBSCRIPTIONS[topic_arn] = [
-            sub for sub in SNS_SUBSCRIPTIONS[topic_arn]
+            sub for sub in existing_subs
             if sub['SubscriptionArn'] != subscription_arn
         ]
 
@@ -486,10 +487,6 @@ def do_untag_resource(topic_arn, tag_keys):
 # ---------------
 
 
-def get_topic_by_arn(topic_arn):
-    return SNS_SUBSCRIPTIONS.get(topic_arn)
-
-
 def get_subscription_by_arn(sub_arn):
     # TODO maintain separate map instead of traversing all items
     for key, subscriptions in SNS_SUBSCRIPTIONS.items():
