@@ -9,7 +9,8 @@ import botocore
 from localstack import config
 from localstack.constants import (
     REGION_LOCAL, LOCALHOST, MOTO_ACCOUNT_ID, ENV_DEV, APPLICATION_AMZ_JSON_1_1,
-    APPLICATION_AMZ_JSON_1_0, APPLICATION_X_WWW_FORM_URLENCODED, TEST_AWS_ACCOUNT_ID)
+    APPLICATION_AMZ_JSON_1_0, APPLICATION_X_WWW_FORM_URLENCODED, TEST_AWS_ACCOUNT_ID,
+    MAX_POOL_CONNECTIONS, TEST_AWS_ACCESS_KEY_ID, TEST_AWS_SECRET_ACCESS_KEY)
 from localstack.utils.aws import templating
 from localstack.utils.common import (
     run_safe, to_str, is_string, is_string_or_bytes, make_http_request, is_port_open, get_service_protocol)
@@ -151,10 +152,10 @@ def get_boto3_credentials():
         return boto3.session.Session().get_credentials()
 
 
-def get_boto3_session():
-    if CUSTOM_BOTO3_SESSION:
+def get_boto3_session(cache=True):
+    if cache and CUSTOM_BOTO3_SESSION:
         return CUSTOM_BOTO3_SESSION
-    if CREATE_NEW_SESSION_PER_BOTO3_CONNECTION:
+    if not cache or CREATE_NEW_SESSION_PER_BOTO3_CONNECTION:
         return boto3.session.Session()
     # return default session
     return boto3
@@ -204,7 +205,7 @@ def connect_to_resource(service_name, env=None, region_name=None, endpoint_url=N
 
 
 def connect_to_service(service_name, client=True, env=None, region_name=None, endpoint_url=None,
-        config=None, verify=False, *args, **kwargs):
+        config=None, verify=False, cache=True, *args, **kwargs):
     """
     Generic method to obtain an AWS service client using boto3, based on environment, region, or custom endpoint_url.
     """
@@ -213,19 +214,30 @@ def connect_to_service(service_name, client=True, env=None, region_name=None, en
     region = env.region if env.region != REGION_LOCAL else region_name
     key_elements = [service_name, client, env, region, endpoint_url, config]
     cache_key = '/'.join([str(k) for k in key_elements])
-    if cache_key not in BOTO_CLIENTS_CACHE:
+    if not cache or cache_key not in BOTO_CLIENTS_CACHE:
         # Cache clients, as this is a relatively expensive operation
-        my_session = get_boto3_session()
+        my_session = get_boto3_session(cache=cache)
         method = my_session.client if client else my_session.resource
         if not endpoint_url:
             if is_local_env(env):
                 endpoint_url = get_local_service_url(service_name)
                 verify = False
+            backend_env_name = '%s_BACKEND' % service_name.upper()
+            backend_url = os.environ.get(backend_env_name, '').strip()
+            if backend_url:
+                endpoint_url = backend_url
         config = config or botocore.client.Config()
-        # prevent error "Connection pool is full, discarding connection ..."
-        config.max_pool_connections = 150
-        BOTO_CLIENTS_CACHE[cache_key] = method(service_name, region_name=region,
+        # configure S3 path style addressing
+        if service_name == 's3':
+            config.s3 = {'addressing_style': 'path'}
+        # To, prevent error "Connection pool is full, discarding connection ...",
+        # set the environment variable MAX_POOL_CONNECTIONS. Default is 150.
+        config.max_pool_connections = MAX_POOL_CONNECTIONS
+        result = method(service_name, region_name=region,
             endpoint_url=endpoint_url, verify=verify, config=config)
+        if not cache:
+            return result
+        BOTO_CLIENTS_CACHE[cache_key] = result
 
     return BOTO_CLIENTS_CACHE[cache_key]
 
@@ -235,6 +247,22 @@ def render_velocity_template(*args, **kwargs):
     return templating.render_velocity_template(*args, **kwargs)
 
 
+def generate_presigned_url(*args, **kwargs):
+    id_before = os.environ.get(ENV_ACCESS_KEY)
+    key_before = os.environ.get(ENV_SECRET_KEY)
+    try:
+        # Note: presigned URL needs to be created with test credentials
+        os.environ[ENV_ACCESS_KEY] = TEST_AWS_ACCESS_KEY_ID
+        os.environ[ENV_SECRET_KEY] = TEST_AWS_SECRET_ACCESS_KEY
+        s3_client = connect_to_service('s3', cache=False)
+        return s3_client.generate_presigned_url(*args, **kwargs)
+    finally:
+        if id_before:
+            os.environ[ENV_ACCESS_KEY] = id_before
+        if key_before:
+            os.environ[ENV_SECRET_KEY] = key_before
+
+
 def check_valid_region(headers):
     """ Check whether a valid region is provided, and if not then raise an Exception. """
     auth_header = headers.get('Authorization')
@@ -279,11 +307,11 @@ def fix_account_id_in_arns(response, colon_delimiter=':', existing=None, replace
     return content
 
 
-def get_s3_client():
-    return boto3.resource('s3',
-        endpoint_url=config.TEST_S3_URL,
-        config=boto3.session.Config(s3={'addressing_style': 'path'}),
-        verify=False)
+def inject_test_credentials_into_env(env):
+    env = env or {}
+    if ENV_ACCESS_KEY not in env and ENV_SECRET_KEY not in env:
+        env[ENV_ACCESS_KEY] = 'test'
+        env[ENV_SECRET_KEY] = 'test'
 
 
 def sqs_queue_url_for_arn(queue_arn):
@@ -329,6 +357,10 @@ def role_arn(role_name, account_id=None, env=None):
     return 'arn:aws:iam::%s:role/%s' % (account_id, role_name)
 
 
+def policy_arn(policy_name, account_id=TEST_AWS_ACCOUNT_ID):
+    return 'arn:aws:iam::{}:policy/{}'.format(account_id, policy_name)
+
+
 def iam_resource_arn(resource, role=None, env=None):
     env = get_environment(env)
     if not role:
@@ -444,6 +476,11 @@ def es_domain_arn(domain_name, account_id=None, region_name=None):
     return _resource_arn(domain_name, pattern, account_id=account_id, region_name=region_name)
 
 
+def code_signing_arn(code_signing_id, account_id=None, region_name=None):
+    pattern = 'arn:aws:lambda:%s:%s:code-signing-config:%s'
+    return _resource_arn(code_signing_id, pattern, account_id=account_id, region_name=region_name)
+
+
 def s3_bucket_arn(bucket_name, account_id=None):
     return 'arn:aws:s3:::%s' % (bucket_name)
 
@@ -540,10 +577,10 @@ def dynamodb_get_item_raw(request):
     return new_item
 
 
-def create_dynamodb_table(table_name, partition_key, env=None, stream_view_type=None):
-    """Utility method to create a DynamoDB table"""
+def create_dynamodb_table(table_name, partition_key, env=None, stream_view_type=None, region_name=None, client=None):
+    """ Utility method to create a DynamoDB table """
 
-    dynamodb = connect_to_service('dynamodb', env=env, client=True)
+    dynamodb = client or connect_to_service('dynamodb', env=env, client=True, region_name=region_name)
     stream_spec = {'StreamEnabled': False}
     key_schema = [{
         'AttributeName': partition_key,
@@ -569,7 +606,9 @@ def create_dynamodb_table(table_name, partition_key, env=None, stream_view_type=
     except Exception as e:
         if 'ResourceInUseException' in str(e):
             # Table already exists -> return table reference
-            return connect_to_resource('dynamodb', env=env).Table(table_name)
+            return connect_to_resource('dynamodb', env=env, region_name=region_name).Table(table_name)
+        if 'AccessDeniedException' in str(e):
+            raise
     time.sleep(2)
     return table
 
