@@ -5,8 +5,11 @@ import yaml
 import logging
 import traceback
 import moto.cloudformation.utils
-from six import iteritems
+
 from urllib.parse import urlparse
+from six import iteritems
+from moto.cloudformation.models import cloudformation_backends
+from moto.cloudformation import parsing
 from localstack.utils import common
 from localstack.utils.aws import aws_stack
 from localstack.constants import AWS_REGION_US_EAST_1
@@ -22,7 +25,7 @@ PLACEHOLDER_AWS_NO_VALUE = '__aws_no_value__'
 LOG = logging.getLogger(__name__)
 
 # list of resource types that can be updated
-UPDATEABLE_RESOURCES = ['Lambda::Function', 'ApiGateway::Method']
+UPDATEABLE_RESOURCES = ['Lambda::Function', 'ApiGateway::Method', 'StepFunctions::StateMachine']
 
 # list of static attribute references to be replaced in {'Fn::Sub': '...'} strings
 STATIC_REFS = ['AWS::Region', 'AWS::Partition', 'AWS::StackName']
@@ -60,6 +63,10 @@ def lambda_get_params():
     return lambda params, **kwargs: params
 
 
+def lambda_keys_to_lower(key=None):
+    return lambda params, **kwargs: common.keys_to_lower(params.get(key) if key else params)
+
+
 def rename_params(func, rename_map):
     def do_rename(params, **kwargs):
         values = func(params, **kwargs) if func else params
@@ -493,7 +500,6 @@ RESOURCE_TO_FUNCTION = {
             'parameters': {
                 'Name': 'PhysicalResourceId'
             }
-
         }
     },
     'IAM::Role': {
@@ -520,6 +526,7 @@ RESOURCE_TO_FUNCTION = {
             # TODO: associate policies with users, groups, roles
             'parameters': iam_create_policy_params
         }
+        # InlinePolicy in cloudformation will be deleted on deleting Role
     },
     'ApiGateway::RestApi': {
         'create': {
@@ -558,6 +565,12 @@ RESOURCE_TO_FUNCTION = {
     },
     'ApiGateway::Account': {
     },
+    'ApiGateway::Stage': {
+        'create': {
+            'function': 'create_stage',
+            'parameters': lambda_keys_to_lower()
+        }
+    },
     'ApiGateway::Deployment': {
         'create': {
             'function': 'create_deployment',
@@ -608,6 +621,12 @@ RESOURCE_TO_FUNCTION = {
                 'roleArn': lambda params, **kwargs: get_role_arn(params.get('RoleArn'), **kwargs)
             }
         },
+        'update': {
+            'function': 'update_state_machine',
+            'parameters': {
+                'definition': 'DefinitionString'
+            }
+        },
         'delete': {
             'function': 'delete_state_machine',
             'parameters': {
@@ -758,6 +777,18 @@ def describe_stack_resource(stack_name, logical_resource_id):
                     (logical_resource_id, stack_name, e))
 
 
+def retrieve_and_update_resource_details(resource_id, resource_status, resources, stack_name):
+    resource = resources.get(resource_id) or {}
+    resource_props = resource.get('Properties') or {}
+    result = retrieve_resource_details(resource_id, resource_status, resources, stack_name)
+    if isinstance(result, dict):
+        result.pop('ResponseMetadata', None)
+        # Note: for now, we're only setting non-existing props (we may set changed props in the future as well)
+        update_attrs = {k: v for k, v in result.items() if k not in resource_props}
+        resource_props.update(update_attrs)
+    return result
+
+
 def retrieve_resource_details(resource_id, resource_status, resources, stack_name):
     resource = resources.get(resource_id)
     resource_id = resource_status.get('PhysicalResourceId') or resource_id
@@ -771,13 +802,14 @@ def retrieve_resource_details(resource_id, resource_status, resources, stack_nam
             resource_id = func_name if resource else resource_id
             return aws_stack.connect_to_service('lambda').get_function(FunctionName=resource_id)
         elif resource_type == 'Lambda::Version':
-            name = resource_props.get('FunctionName')
+            name = resolve_refs_recursively(stack_name, resource_props.get('FunctionName'), resources)
             if not name:
                 return None
             func_name = aws_stack.lambda_function_name(name)
             func_version = name.split(':')[7] if len(name.split(':')) > 7 else '$LATEST'
             versions = aws_stack.connect_to_service('lambda').list_versions_by_function(FunctionName=func_name)
             return ([v for v in versions['Versions'] if v['Version'] == func_version] or [None])[0]
+
         elif resource_type == 'Lambda::EventSourceMapping':
             resource_id = resource_props['FunctionName'] if resource else resource_id
             source_arn = resource_props.get('EventSourceArn')
@@ -793,24 +825,33 @@ def retrieve_resource_details(resource_id, resource_status, resources, stack_nam
             if not mapping:
                 raise Exception('ResourceNotFound')
             return mapping[0]
+
+        elif resource_type == 'Events::Rule':
+            rule_name = resolve_refs_recursively(stack_name, resource_props.get('Name'), resources)
+            result = aws_stack.connect_to_service('events').describe_rule(Name=rule_name) or {}
+            return result if result.get('Name') else None
+
         elif resource_type == 'IAM::Role':
-            role_name = resource_props.get('RoleName')
-            role_name = resolve_refs_recursively(stack_name, role_name, resources)
+            role_name = resolve_refs_recursively(stack_name, resource_props.get('RoleName'), resources)
             return aws_stack.connect_to_service('iam').get_role(RoleName=role_name)['Role']
+
         elif resource_type == 'SSM::Parameter':
             param_name = resource_props.get('Name') or resource_id
             param_name = resolve_refs_recursively(stack_name, param_name, resources)
             return aws_stack.connect_to_service('ssm').get_parameter(Name=param_name)['Parameter']
+
         elif resource_type == 'DynamoDB::Table':
             table_name = resource_props.get('TableName') or resource_id
             table_name = resolve_refs_recursively(stack_name, table_name, resources)
             return aws_stack.connect_to_service('dynamodb').describe_table(TableName=table_name)
+
         elif resource_type == 'ApiGateway::RestApi':
             apis = aws_stack.connect_to_service('apigateway').get_rest_apis()['items']
-            api_name = resource_props['Name'] if resource else resource_id
+            api_name = resource_props.get('Name') or resource_id
             api_name = resolve_refs_recursively(stack_name, api_name, resources)
             result = list(filter(lambda api: api['name'] == api_name, apis))
             return result[0] if result else None
+
         elif resource_type == 'ApiGateway::Resource':
             api_id = resource_props['RestApiId'] if resource else resource_id
             api_id = resolve_refs_recursively(stack_name, api_id, resources)
@@ -826,6 +867,7 @@ def retrieve_resource_details(resource_id, resource_status, resources, stack_nam
                 target_resource[0]['id'], resources=api_resources)
             result = list(filter(lambda res: res['path'] == path, api_resources))
             return result[0] if result else None
+
         elif resource_type == 'ApiGateway::Deployment':
             api_id = resource_props['RestApiId'] if resource else resource_id
             api_id = resolve_refs_recursively(stack_name, api_id, resources)
@@ -834,6 +876,16 @@ def retrieve_resource_details(resource_id, resource_status, resources, stack_nam
             result = aws_stack.connect_to_service('apigateway').get_deployments(restApiId=api_id)['items']
             # TODO possibly filter results by stage name or other criteria
             return result[0] if result else None
+
+        elif resource_type == 'ApiGateway::Stage':
+            api_id = resource_props['RestApiId'] if resource else resource_id
+            api_id = resolve_refs_recursively(stack_name, api_id, resources)
+            if not api_id:
+                return None
+            result = aws_stack.connect_to_service('apigateway').get_stage(restApiId=api_id,
+                stageName=resource_props['StageName'])
+            return result
+
         elif resource_type == 'ApiGateway::Method':
             api_id = resolve_refs_recursively(stack_name, resource_props['RestApiId'], resources)
             res_id = resolve_refs_recursively(stack_name, resource_props['ResourceId'], resources)
@@ -848,26 +900,32 @@ def retrieve_resource_details(resource_id, resource_status, resources, stack_nam
                     m.get('methodIntegration', {}).get('type') == 'AWS_PROXY' and
                     m.get('methodIntegration', {}).get('httpMethod') == int_props.get('IntegrationHttpMethod')]
             return any(match) or None
+
         elif resource_type == 'ApiGateway::GatewayResponse':
             api_id = resolve_refs_recursively(stack_name, resource_props['RestApiId'], resources)
             client = aws_stack.connect_to_service('apigateway')
             result = client.get_gateway_response(restApiId=api_id, responseType=resource_props['ResponseType'])
             return result if 'responseType' in result else None
+
         elif resource_type == 'SQS::Queue':
+            queue_name = resolve_refs_recursively(stack_name, resource_props['QueueName'], resources)
             sqs_client = aws_stack.connect_to_service('sqs')
             queues = sqs_client.list_queues()
             result = list(filter(lambda item:
                 # TODO possibly find a better way to compare resource_id with queue URLs
-                item.endswith('/%s' % resource_id), queues.get('QueueUrls', [])))
+                item.endswith('/%s' % queue_name), queues.get('QueueUrls', [])))
             if not result:
                 return None
             result = sqs_client.get_queue_attributes(QueueUrl=result[0], AttributeNames=['All'])['Attributes']
             result['Arn'] = result['QueueArn']
             return result
+
         elif resource_type == 'SNS::Topic':
+            topic_name = resolve_refs_recursively(stack_name, resource_props['TopicName'], resources)
             topics = aws_stack.connect_to_service('sns').list_topics()
-            result = list(filter(lambda item: item['TopicArn'] == resource_id, topics.get('Topics', [])))
+            result = list(filter(lambda item: item['TopicArn'].split(':')[-1] == topic_name, topics.get('Topics', [])))
             return result[0] if result else None
+
         elif resource_type == 'SNS::Subscription':
             topic_arn = resource_props.get('TopicArn')
             topic_arn = resolve_refs_recursively(stack_name, topic_arn, resources)
@@ -877,6 +935,7 @@ def retrieve_resource_details(resource_id, resource_status, resources, stack_nam
                 resource_props.get('Endpoint') == sub['Endpoint']]
             # TODO: use get_subscription_attributes to compare FilterPolicy
             return result[0] if result else None
+
         elif resource_type == 'S3::Bucket':
             bucket_name = resource_props.get('BucketName') or resource_id
             bucket_name = resolve_refs_recursively(stack_name, bucket_name, resources)
@@ -892,20 +951,24 @@ def retrieve_resource_details(resource_id, resource_status, resources, stack_nam
             if notifs and not has_notifs:
                 return None
             return response
+
         elif resource_type == 'S3::BucketPolicy':
             bucket_name = resource_props.get('Bucket') or resource_id
             bucket_name = resolve_refs_recursively(stack_name, bucket_name, resources)
             return aws_stack.connect_to_service('s3').get_bucket_policy(Bucket=bucket_name)
+
         elif resource_type == 'Logs::LogGroup':
             group_name = resource_props.get('LogGroupName')
             group_name = resolve_refs_recursively(stack_name, group_name, resources)
             logs = aws_stack.connect_to_service('logs')
             groups = logs.describe_log_groups(logGroupNamePrefix=group_name)['logGroups']
             return ([g for g in groups if g['logGroupName'] == group_name] or [None])[0]
+
         elif resource_type == 'Kinesis::Stream':
             stream_name = resolve_refs_recursively(stack_name, resource_props['Name'], resources)
             result = aws_stack.connect_to_service('kinesis').describe_stream(StreamName=stream_name)
             return result
+
         elif resource_type == 'StepFunctions::StateMachine':
             sm_name = resource_props.get('StateMachineName') or resource_id
             sm_name = resolve_refs_recursively(stack_name, sm_name, resources)
@@ -916,6 +979,7 @@ def retrieve_resource_details(resource_id, resource_status, resources, stack_nam
                 return None
             result = sfn_client.describe_state_machine(stateMachineArn=sm_arn[0])
             return result
+
         elif resource_type == 'StepFunctions::Activity':
             act_name = resource_props.get('Name') or resource_id
             act_name = resolve_refs_recursively(stack_name, act_name, resources)
@@ -925,57 +989,114 @@ def retrieve_resource_details(resource_id, resource_status, resources, stack_nam
             if not result:
                 return None
             return result[0]
+
         elif resource_type == 'SecretsManager::Secret':
             secret_name = resource_props.get('Name') or resource_id
             secret_name = resolve_refs_recursively(stack_name, secret_name, resources)
             return aws_stack.connect_to_service('secretsmanager').describe_secret(SecretId=secret_name)
+
         elif resource_type == 'Elasticsearch::Domain':
             domain_name = resource_props.get('DomainName') or resource_id
             domain_name = resolve_refs_recursively(stack_name, domain_name, resources)
             return aws_stack.connect_to_service('es').describe_elasticsearch_domain(DomainName=domain_name)
+
         elif resource_type == 'KinesisFirehose::DeliveryStream':
             stream_name = resource_props.get('DeliveryStreamName') or resource_id
             stream_name = resolve_refs_recursively(stack_name, stream_name, resources)
             return aws_stack.connect_to_service('firehose').describe_delivery_stream(DeliveryStreamName=stream_name)
+
+        elif resource_type == 'IAM::Policy':
+            client = aws_stack.connect_to_service('iam')
+            policy_name = resource_props.get('PolicyName') or resource_id
+
+            # The policy in cloudformation is InlinePolicy, so we must specify at least one of [Roles, Users, Groups]
+            # https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iam-policy.html
+            roles = resource['Properties'].get('Roles', [])
+            if roles:
+                if isinstance(roles[0], str):
+                    role_name = roles[0]
+                else:
+                    role_name = resources[roles[0]['Ref']]['Properties']['RoleName']
+
+                role_policy = client.get_role_policy(RoleName=role_name, PolicyName=policy_name)
+                return role_policy
+
+            # TODO The InlinePolicy can be embedded in specified User, or Group
+
         if is_deployable_resource(resource):
             LOG.warning('Unexpected resource type %s when resolving references of resource %s: %s' %
                         (resource_type, resource_id, resource))
+
     except Exception as e:
         check_not_found_exception(e, resource_type, resource, resource_status)
+
     return None
 
 
 def check_not_found_exception(e, resource_type, resource, resource_status):
     # we expect this to be a "not found" exception
-    markers = ['NoSuchBucket', 'ResourceNotFound', '404', 'not found']
+    markers = ['NoSuchBucket', 'ResourceNotFound', 'NotFoundException', '404', 'not found']
     if not list(filter(lambda marker, e=e: marker in str(e), markers)):
         LOG.warning('Unexpected error retrieving details for resource %s: %s %s - %s %s' %
             (resource_type, e, traceback.format_exc(), resource, resource_status))
 
 
-def extract_resource_attribute(resource_type, resource, attribute):
+def extract_resource_attribute(resource_type, resource_json, attribute, resource_id=None, resource=None):
     LOG.debug('Extract resource attribute: %s %s' % (resource_type, attribute))
     # extract resource specific attributes
     if resource_type == 'Lambda::Function':
-        actual_attribute = 'FunctionArn' if attribute == 'Arn' else attribute
-        return resource['Configuration'][actual_attribute]
+        func_configs = resource_json.get('Configuration')
+        if attribute in ['Arn', 'PhysicalResourceId']:
+            if isinstance(resource, dict):
+                func_configs = resource.get('Properties', {})
+            return func_configs.get('FunctionArn') or aws_stack.lambda_function_arn(func_configs.get('FunctionName'))
+        return func_configs.get(attribute)
     elif resource_type == 'DynamoDB::Table':
         actual_attribute = 'LatestStreamArn' if attribute == 'StreamArn' else attribute
-        value = resource['Table'].get(actual_attribute)
+        value = resource_json['Table'].get(actual_attribute)
         return value
     elif resource_type == 'ApiGateway::RestApi':
         if attribute == 'PhysicalResourceId':
-            return resource['id']
+            return resource_json['id']
         if attribute == 'RootResourceId':
-            resources = aws_stack.connect_to_service('apigateway').get_resources(restApiId=resource['id'])['items']
+            resources = aws_stack.connect_to_service('apigateway').get_resources(restApiId=resource_json['id'])['items']
             for res in resources:
                 if res['path'] == '/' and not res.get('parentId'):
                     return res['id']
     elif resource_type == 'ApiGateway::Resource':
         if attribute == 'PhysicalResourceId':
-            return resource['id']
+            return resource_json['id']
+    elif resource_type == 'S3::Bucket':
+        if attribute == 'PhysicalResourceId' and isinstance(resource, dict):
+            return resource.get('Properties', {}).get('BucketName')
+    elif resource_type == 'SNS::Topic':
+        if attribute == 'PhysicalResourceId' and resource_json.get('TopicArn'):
+            return resource_json.get('TopicArn')
     attribute_lower = common.first_char_to_lower(attribute)
-    return resource.get(attribute) or resource.get(attribute_lower)
+    result = resource_json.get(attribute) or resource_json.get(attribute_lower)
+    if result is None and isinstance(resource, dict):
+        res_json1 = resource.get('Properties', {})
+        result = res_json1.get(attribute) or res_json1.get(attribute_lower)
+        if result is None:
+            result = get_attr_from_model_instance(resource, attribute,
+                resource_type=resource_type, resource_id=resource_id)
+    if result is None and attribute == 'PhysicalResourceId':
+        result = resource_json.get('Id')
+    return result
+
+
+def get_attr_from_model_instance(resource, attribute, resource_type, resource_id=None):
+    if not resource_type.startswith('AWS::'):
+        resource_type = 'AWS::%s' % resource_type
+    model_clazz = parsing.MODEL_MAP.get(resource_type)
+    if not model_clazz:
+        LOG.info('Unable to find model class for resource type "%s"' % resource_type)
+        return
+    try:
+        inst = model_clazz(resource_name=resource_id, resource_json=resource)
+        return inst.get_cfn_attribute(attribute)
+    except Exception:
+        pass
 
 
 def resolve_ref(stack_name, ref, resources, attribute):
@@ -995,60 +1116,90 @@ def resolve_ref(stack_name, ref, resources, attribute):
 
     # second, resolve resource references
     resource_status = {}
-    if stack_name:
-        resource_status = describe_stack_resource(stack_name, ref)
-        if not resource_status:
-            return
-        attr_value = resource_status.get(attribute)
-        if attr_value not in [None, '']:
-            return attr_value
-    elif ref in resources:
-        resource_status = resources[ref]['__details__']
+
+    # TODO: check if the logic below (calling describe_stack_resource during deployment) is still required!
+    # if stack_name:
+    #     resource_status = describe_stack_resource(stack_name, ref)
+    #     if not resource_status:
+    #         return
+    #     attr_value = resource_status.get(attribute)
+    #     if attr_value not in [None, '']:
+    #         return attr_value
+
+    if not resource_status and resources.get(ref):
+        resource_status = resources[ref].get('__details__', {})
+    if not resource_status and resources.get(ref):
+        if isinstance(resources[ref].get(attribute), (str, int, float, bool, dict)):
+            return resources[ref][attribute]
     # fetch resource details
     resource_new = retrieve_resource_details(ref, resource_status, resources, stack_name)
     if not resource_new:
         return
     resource = resources.get(ref)
     resource_type = get_resource_type(resource)
-    result = extract_resource_attribute(resource_type, resource_new, attribute)
+    result = extract_resource_attribute(resource_type, resource_new, attribute, resource_id=ref, resource=resource)
     if not result:
-        LOG.warning('Unable to extract reference attribute %s from resource: %s' % (attribute, resource_new))
+        LOG.warning('Unable to extract reference attribute %s from resource: %s %s' %
+            (attribute, resource_new, resource))
     return result
 
 
 def resolve_refs_recursively(stack_name, value, resources):
     if isinstance(value, dict):
         keys_list = list(value.keys())
+
         # process special operators
         if keys_list == ['Ref']:
-            return resolve_ref(stack_name, value['Ref'],
-                resources, attribute='PhysicalResourceId')
+            return resolve_ref(stack_name, value['Ref'], resources, attribute='PhysicalResourceId')
+
         if keys_list and keys_list[0].lower() == 'fn::getatt':
-            return resolve_ref(stack_name, value[keys_list[0]][0],
-                resources, attribute=value[keys_list[0]][1])
+            return resolve_ref(stack_name, value[keys_list[0]][0], resources, attribute=value[keys_list[0]][1])
+
         if keys_list and keys_list[0].lower() == 'fn::join':
             join_values = value[keys_list[0]][1]
             join_values = [resolve_refs_recursively(stack_name, v, resources) for v in join_values]
             none_values = [v for v in join_values if v is None]
             if none_values:
                 raise Exception('Cannot resolve CF fn::Join %s due to null values: %s' % (value, join_values))
+
             return value[keys_list[0]][0].join(join_values)
+
         if keys_list and keys_list[0].lower() == 'fn::sub':
             item_to_sub = value[keys_list[0]]
+
             if not isinstance(item_to_sub, list):
                 attr_refs = dict([(r, {'Ref': r}) for r in STATIC_REFS])
                 item_to_sub = [item_to_sub, attr_refs]
             result = item_to_sub[0]
+
             for key, val in item_to_sub[1].items():
                 val = resolve_refs_recursively(stack_name, val, resources)
                 result = result.replace('${%s}' % key, val)
+
             return result
+
+        if keys_list and keys_list[0].lower() == 'fn::findinmap':
+            result = resolve_ref(stack_name, value[keys_list[0]][0], resources, attribute=value[keys_list[0]][1])
+            if not result:
+                raise Exception('Cannot resolve fn::FindInMap: %s %s' % (value[keys_list[0]], list(resources.keys())))
+
+            result = result.get(value[keys_list[0]][2])
+            return result
+
+        if keys_list and keys_list[0].lower() == 'fn::importvalue':
+            exports = cloudformation_backends[aws_stack.get_region()].exports
+            import_value_key = resolve_refs_recursively(stack_name, value[keys_list[0]], resources)
+            export = exports[import_value_key]
+            return export.value
+
         else:
             for key, val in iteritems(value):
                 value[key] = resolve_refs_recursively(stack_name, val, resources)
+
     if isinstance(value, list):
         for i in range(0, len(value)):
             value[i] = resolve_refs_recursively(stack_name, value[i], resources)
+
     return value
 
 
@@ -1106,6 +1257,15 @@ def update_resource(resource_id, resources, stack_name):
 
         return client.put_method(**kwargs)
 
+    if resource_type == 'StepFunctions::StateMachine':
+        client = aws_stack.connect_to_service('stepfunctions')
+        kwargs = {
+            'stateMachineArn': props['stateMachineArn'],
+            'definition': props['DefinitionString'],
+        }
+
+        return client.update_state_machine(**kwargs)
+
 
 def fix_account_id_in_arns(params):
     def fix_ids(o, **kwargs):
@@ -1250,7 +1410,8 @@ def configure_resource_via_sdk(resource_id, resources, resource_type, func_detai
                     params[param_key] = PLACEHOLDER_RESOURCE_NAME
                 else:
                     if callable(prop_key):
-                        prop_value = prop_key(resource_props, stack_name=stack_name, resources=resources)
+                        prop_value = prop_key(resource_props, stack_name=stack_name,
+                            resources=resources, resource_id=resource_id)
                     else:
                         prop_value = resource_props.get(prop_key)
                     if prop_value is not None:
@@ -1370,7 +1531,6 @@ def delete_stack(stack_name, stack_resources):
     resources = dict([(r['LogicalResourceId'], common.clone_safe(r)) for r in stack_resources])
     for key, resource in resources.items():
         resources[key]['Properties'] = common.clone_safe(resource)
-
     for resource_id in resources.keys():
         delete_resource(resource_id, resources, stack_name)
 
@@ -1378,26 +1538,36 @@ def delete_stack(stack_name, stack_resources):
 # --------
 # Util methods for analyzing resource dependencies
 # --------
+
 def is_deployable_resource(resource):
     resource_type = get_resource_type(resource)
     entry = RESOURCE_TO_FUNCTION.get(resource_type)
     if entry is None:
-        LOG.warning('Unknown resource type "%s": %s' % (resource_type, resource))
+        LOG.warning('Unknown resource type "%s" in resource deployment map: %s' % (resource_type, resource))
     return bool(entry and entry.get(ACTION_CREATE))
 
 
+def get_deployment_state(resource_id, resources, stack_name):
+    res_details = resources[resource_id]
+    resource_status = res_details.get('__details__') or {}
+    details = retrieve_and_update_resource_details(resource_id, resource_status, resources, stack_name)
+    return details
+
+
 def is_deployed(resource_id, resources, stack_name):
-    resource = resources[resource_id]
-    resource_status = resource.get('__details__') or {}
-    details = retrieve_resource_details(resource_id, resource_status, resources, stack_name)
+    details = get_deployment_state(resource_id, resources, stack_name)
     return bool(details)
 
 
-def should_be_deployed(resource_id, resources, stack_name):
+def should_be_deployed(resource_id, resources, stack_name, deploy_state=None):
     """ Return whether the given resource is all of: (1) deployable, (2) not yet deployed,
         and (3) has no unresolved dependencies. """
-    resource = resources[resource_id]
-    if not is_deployable_resource(resource) or is_deployed(resource_id, resources, stack_name):
+    res_details = resources[resource_id]
+    if not is_deployable_resource(res_details):
+        return False
+    if deploy_state is None:
+        deploy_state = is_deployed(resource_id, resources, stack_name)
+    if deploy_state:
         return False
     return all_resource_dependencies_satisfied(resource_id, resources, stack_name)
 
@@ -1412,20 +1582,31 @@ def is_updateable(resource_id, resources, stack_name):
 
 
 def all_resource_dependencies_satisfied(resource_id, resources, stack_name):
+    unsatisfied = get_unsatisfied_dependencies(resource_id, resources, stack_name)
+    return not unsatisfied
+
+
+def get_unsatisfied_dependencies(resource_id, resources, stack_name):
     resource = resources[resource_id]
     res_deps = get_resource_dependencies(resource_id, resource, resources)
-    return all_dependencies_satisfied(res_deps, stack_name, resources, resource_id)
+    return get_unsatisfied_dependencies_for_resources(res_deps, stack_name, resources, resource_id)
 
 
-def all_dependencies_satisfied(resources, stack_name, all_resources, depending_resource=None):
+def get_unsatisfied_dependencies_for_resources(
+        resources, stack_name, all_resources, depending_resource=None, return_first=True):
+    result = {}
     for resource_id, resource in iteritems(resources):
         if is_deployable_resource(resource):
             if not is_deployed(resource_id, all_resources, stack_name):
-                LOG.debug('Dependency for resource %s not yet deployed: %s' % (depending_resource, resource_id))
-                return False
-    return True
+                LOG.debug('Dependency for resource %s not yet deployed: %s %s' %
+                    (depending_resource, resource_id, resource))
+                result[resource_id] = resource
+                if return_first:
+                    break
+    return result
 
 
+# TODO: check if still needed
 def resources_to_deploy_next(resources, stack_name):
     result = {}
     for resource_id, resource in resources.items():
@@ -1448,4 +1629,5 @@ def get_resource_dependencies(resource_id, resource, resources):
                 result[other_id] = other
             if other_id in dependencies:
                 result[other_id] = other
+
     return result
